//////////////////////////////////////////////////////////////////////
//
// File: smbuser.c
//
// Description: C test controller program for smblite
//
// Copyright (c) 2002 TiVo Inc.
//
//////////////////////////////////////////////////////////////////////

#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <string.h>
#include <signal.h>
#include <stdlib.h>

#include <util/tvtypes.h>

//#define BUFF_SIZE ( 64512 * 2 )
#define BUFF_SIZE ( 16 * 1024 * 3 )

// MFK: these need to move to an instance struct
static char bufferG[ BUFF_SIZE ];
static char* pBufferG = bufferG;
static int fTerminate = 0;
static const char* pPartialCommand = NULL;
static int fQueuedCommands = 1;
static int dataFd;

static char* GetString( int fd )
{
    char* pString;
    char sizeBuf[4];
    U16 sString;

    if ( read( fd, sizeBuf, 4 ) != 4 ||
         sscanf( sizeBuf, "%4hx", &sString ) != 1 ) {
        fTerminate = 1;
        return NULL;
    }

    pString = malloc( sString + 1 );
    if ( pString == NULL ) {
        return NULL;
    }

    if ( sString != 0 ) {
        if ( read( fd, pString, sString ) != sString ) {
            free ( pString );
            return NULL;
        }
    }

    pString[ sString ] = 0;

    return pString;
}

static void
ProcessOneAck( int fd )
{
    int nRead;
    char ackBuffer[5];
    U16 hCommand;

    nRead = read( fd, ackBuffer, 5  );
    if ( nRead < 5 ) {
        fTerminate = 1;
        return;
    }
#if 1 // MFK debug
    fwrite( ackBuffer, 1, nRead, stderr );
#endif

    if ( sscanf( ackBuffer + 1, "%4hx", &hCommand ) != 1 ) {
        fTerminate = 1;
        return;
    }

    switch ( ackBuffer[0] ) {
      case 'U':
        pBufferG = bufferG;
        // fall through
      case 'V':
        {
            size_t sData;
            char readAckBuffer[8];

            sData = 0;
            if ( read( fd, readAckBuffer, 8 ) != 8 ||
                 sscanf( readAckBuffer, "%8x", &sData ) != 1 ) {
                fTerminate = 1;
                break;
            }
#if 1 // MFK debug
            fprintf( stderr, "%08x", sData );
#endif
            if ( sData != 0 ) {
                ssize_t nRead;
                size_t nReadTot = 0;

                while ( nReadTot < sData ) {
                    nRead = read( dataFd, pBufferG, sData - nReadTot );
                    if ( nRead <= 0 ) {
                        fTerminate = 1;
                        break;
                    }
                    nReadTot += nRead;
                    pBufferG += nRead;
                }
#if 1 // MFK debug
                fwrite( pBufferG - nReadTot, 1, nReadTot, stdout );
#endif
                // MFK: for pipe_aligned transport, need to round up to
                // pBufferG to block size
            }
        }
        break;

      case 'W':
#if 0 // MFK perf debug
        {
            struct timeval tv;
            gettimeofday( &tv, NULL );
            fprintf( stderr, "[%d.%03d]", tv.tv_sec % 1000, tv.tv_usec / 1000 );
        }
#endif
        break;

      case 'R':
        {
            char errorAckBuffer[4];
            U16 nError;

            if ( read( fd, errorAckBuffer, 4 ) != 4 ||
                 sscanf( errorAckBuffer, "%4hx", &nError ) != 1 ) {
                fTerminate = 1;
            } else {
#if 1 // MFK debug
                fprintf( stderr, "%04x", nError );
#endif
                // do something...
            }
        }
        break;

      case 'K':
        break;

      case 'P':
        {
            char cancelAckBuffer[4];
            U16 hCanceled;

            if ( read( fd, cancelAckBuffer, 4 ) != 4 ||
                 sscanf( cancelAckBuffer, "%4hx", &hCanceled ) != 1 ) {
                fTerminate = 1;
            } else {
#if 1 // MFK debug
                fprintf( stderr, "%04x", hCanceled );
#endif
                // do something...
            }
        }
        break;

      case 'S':
        {
            U16 attr;
            size_t size;
            time_t cTime;
            time_t aTime;
            time_t mTime;
            char statAckBuffer[36];

            if ( read( fd, statAckBuffer, 36 ) == 36 &&
                 sscanf( statAckBuffer, "%4hx%8x%8lx%8lx%8lx", &attr,
                         &size, &cTime, &aTime, &mTime ) == 5 ) {
#if 1 // MFK debug
                fprintf( stderr, "%04x%08x%08lx%08lx%08lx", attr, size, cTime,
                         aTime, mTime );
#endif
                // MFK: do something...
            } else {
                fTerminate = 1;
            }
        }
        break;

      case 'H':
        {
            char* pShare;
            char* pComment;

            if ( ( pShare = GetString( fd ) ) != NULL &&
                 ( pComment = GetString( fd ) ) != NULL ) {
#if 1 // MFK debug
                fprintf( stderr, "[%s][%s]", pShare, pComment );
#endif
                // MFK: do something...
                free( pShare );
                free( pComment );
            } else {
                fTerminate = 1;
            }
        }
        break;

      case 'I':
        {
            U16 attr;
            size_t size;
            time_t cTime;
            time_t aTime;
            time_t mTime;
            char* pFilename;
            char listAckBuffer[36];

            if ( read( fd, listAckBuffer, 36 ) == 36 &&
                 sscanf( listAckBuffer, "%4hx%8x%8lx%8lx%8lx", &attr,
                         &size, &cTime, &aTime, &mTime ) == 5 &&
                 ( pFilename = GetString( fd ) ) != NULL ) {
#if 1 // MFK debug
                fprintf( stderr, "%04x%08x%08lx%08lx%08lx[%s]", attr, size,
                         cTime, aTime, mTime, pFilename );
#endif
                // MFK: do something...
                free( pFilename );
            } else {
                fTerminate = 1;
            }
        }
        break;

      case 'T':
        {
            char* pService;

            if ( ( pService = GetString( fd ) ) != NULL ) {
#if 1 // MFK debug
                fprintf( stderr, "[%s]", pService );
#endif
                free( pService );
            } else {
                fTerminate = 1;
            }
        }
        break;

      case 'Q':
        {
            char queryAckBuffer[8];
            U32 address;

            // MFK: these should probably all retry until nRead == 0...
            if ( read( fd, queryAckBuffer, 8 ) != 8 ||
                 sscanf( queryAckBuffer, "%8lx", &address ) != 1 ) {
                fTerminate = 1;
            } else {
#if 1 // MFK debug
                fprintf( stderr, "%08lx", address );
#endif
                // do something...
            }
        }
        break;

      default:
#if 1 // MFK debug
        fprintf( stderr, "Terminate on unknown ack\n" );
#endif
        fTerminate = 1;
        break;
    }
#if 1 // MFK debug
    fprintf( stderr, "\n" );
#endif
}

static int iCommand = 0;
static char* testCommands[] = {
//    "U00000001f8000000fc0000020001",
//    "U00000000fc0000007e0000020001",
    "U00000000c0000000400000020001",
    "Qfffe",
//    "Tffffc0a81521",
    "Tffff7f000001",
    "N0001c0a815210005PPAN100000005GUEST00000000",
//    "N00017f0000010005PPAN100000005GUEST00000000",
    "H0002",
    "N0003c0a815210005PPAN10004test0005GUEST00000000",
//    "N00037f0000010005PPAN10004test0005GUEST00000000",
    "M0004",
    "Ifffd0005\\tc\\*",
    "Ifffc0004\\foo",
//    "O000200010012tc/index-long.html",
    "O00020001001c\\patch-2.4.19-pre2.bits.diff",
    "Sfff3",
    "R000300004000",
    "R000400004000",
    "R000500004000",
    "R000600004000",
    "R000700004000",
    "R000800004000",
    "R000900004000",
    "R000a00004000",
    "R000b00004000",
    "R000c00004000",
    "R000d00004000",
    "R000e00004000",
    "R000f00004000",
    "R001000004000",
    "R001100004000",
    "R001200004000",
    "R001300004000",
    "R001400004000",
    "R001500004000",
    "R001600004000",
    "R001700004000",
    "R001800004000",
    "R001900004000",
    "R001a00004000",
    "R001b00004000",
    "R001c00004000",
    "R001d00004000",
    "R001e00004000",
    "R001f00004000",
    "R002000004000",
    "R002100004000",
    "R002200004000",
    "R002300004000",
    "R002400004000",
    "R002500004000",
    "R002600004000",
    "R002700004000",
    "R002800004000",
    "R002900004000",
    "R002a00004000",
    "R002b00004000",
    "R002c00004000",
    "R002d00004000",
    "R002e00004000",
    "R002f00004000",
    "K003000000000",
#if 0
    "P0010",
    "R00120000fc00",
    "R00130000fc00",
    "R00140000fc00",
#endif
    "L0015",
    "M0016",
    "X0017",
};

static void
SendOneQueuedCommand( int fd )
{
    int nWritten, lenCommand;
    const char* command;

    if ( pPartialCommand == NULL ) {
        command = testCommands[ iCommand ];
    } else {
        command = pPartialCommand;
    }
    lenCommand = strlen( command );
    nWritten = write( fd, command, lenCommand );
    if ( nWritten <= 0 ) {
        fTerminate = 1;
        return;
    }

    if ( nWritten < lenCommand ) {
        pPartialCommand = command + nWritten;
    } else {
        pPartialCommand = NULL;
        iCommand++;
#if 1 // MFK test
        if ( iCommand == 57 ) {
            iCommand = 11;
        }
#endif
        if ( iCommand >= ( sizeof( testCommands ) / sizeof( testCommands[0] ) ) ) {
            fQueuedCommands = 0;
        }
    }
}

static void
PollFds( int outFd, int inFd )
{
    fd_set readFds, writeFds;

    FD_ZERO( &readFds );
    FD_ZERO( &writeFds );
    FD_SET( inFd, &readFds );
//    FD_SET( dataFd, &readFds );
    if ( fQueuedCommands != 0 ) {
        FD_SET( outFd, &writeFds );
    }

    select( dataFd + 1, &readFds, &writeFds, NULL, NULL );
    if ( FD_ISSET( inFd, &readFds ) ) {
        ProcessOneAck( inFd );
    }
    if ( fQueuedCommands != 0 && FD_ISSET( outFd, &writeFds ) ) {
        SendOneQueuedCommand( outFd );
    }
}

static void
SendCommands( int outFd, int inFd )
{
    while ( fTerminate == 0 ) {
        PollFds( outFd, inFd );
    }
}

int main ( int argc, char **argv )
{
    int pid;
    int to_smblite[2];
    int from_smblite[2];
    int data_smblite[2];
    char dataFdStr[20];

    if ( pipe( to_smblite ) || pipe( from_smblite ) || pipe( data_smblite ) ) {
        perror( "pipe" );
        return 1;
    }

    if ( fcntl( to_smblite[0], F_SETFD, FD_CLOEXEC ) ||
         fcntl( to_smblite[1], F_SETFD, FD_CLOEXEC ) ||
         fcntl( to_smblite[1], F_SETFL, O_NONBLOCK ) ||
         fcntl( from_smblite[0], F_SETFD, FD_CLOEXEC ) ||
         fcntl( from_smblite[1], F_SETFD, FD_CLOEXEC ) ||
         fcntl( data_smblite[0], F_SETFD, FD_CLOEXEC ) ) {
        perror( "fcntl" );
        return 1;
    }

    snprintf( dataFdStr, 20, "%d", data_smblite[1] );
    dataFdStr[19] = 0;

    pid = fork();
    if ( pid < 0 ) {
        perror( "fork" );
        return 1;
    }

    if ( pid == 0 ) {
        dup2( to_smblite[0], 0 );
        dup2( from_smblite[1], 1 );
        execl( "./bin/smblite", "./bin/smblite", dataFdStr, NULL );
        perror( "execl" );
        return 1;
    } else {
        close( to_smblite[0] );
        close( from_smblite[1] );
        close( data_smblite[1] );
    }

    dataFd = data_smblite[0];
    SendCommands( to_smblite[1], from_smblite[0] );
    kill( pid, SIGTERM );
    waitpid( pid, NULL, 0 );

    return 0;
}
